package org.springblade.modules.system.controller;

import java.io.File;
import java.io.IOException;
import java.util.*;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import org.apache.commons.compress.utils.Lists;
import org.springblade.core.redis.cache.BladeRedis;
import org.springblade.core.tool.api.R;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.SetOperations;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import cn.hutool.core.io.FileUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 文件上传
 */
@RestController
@Api(value = "附件管理controller", tags = {"附件管理接口"})
@RequestMapping("/file")
public class FileController {

    @Value("${blade.file.uploadPath}")
    private String picDir;
    @Autowired
    private BladeRedis bladeRedis;

    @PostMapping("/uploadFile")
    @ApiOperation(value = "上传附件", notes = "上传附件，不大于100M")
    public R uploadFile(@RequestParam(value = "serialNumber", required = false) String serialNumber, MultipartFile file) {
        if (null == file) {
            return R.fail("文件不能为空");
        }
        String oFileName = file.getOriginalFilename();
        String fileSuffix = oFileName.substring(oFileName.lastIndexOf("."));
        if (!".jpg".equalsIgnoreCase(fileSuffix) && !".jpeg".equalsIgnoreCase(fileSuffix) && !".gif".equalsIgnoreCase(fileSuffix) && !".png".equalsIgnoreCase(fileSuffix) && !".pdf".equalsIgnoreCase(fileSuffix) && !".doc".equalsIgnoreCase(fileSuffix) && !".docx".equalsIgnoreCase(fileSuffix) && !".xls".equalsIgnoreCase(fileSuffix) && !".jar".equalsIgnoreCase(fileSuffix) && !".xlsx".equalsIgnoreCase(fileSuffix)) {

            return R.fail("只能上传jpg/jpeg/gif/png/pdf/doc/xls/jar类型的文件");
        }

        if (file.getSize() > 1024 * 1000 * 500) {
            return R.fail("附件不得大于500M");
        }

        String tempFileName = UUID.randomUUID().toString() + fileSuffix;

        File f = new File(picDir + tempFileName);
        try {
            FileUtil.writeFromStream(file.getInputStream(), f);
//            FileUtils.copyInputStreamToFile(file.getInputStream(), f);
        } catch (IOException e) {
            e.printStackTrace();
        }

        if (StringUtils.isNotBlank(serialNumber)) {
            SetOperations<String, Object> setOps = bladeRedis.getSetOps();
//            Set<Object> members = setOps.members(serialNumber);
            setOps.add(serialNumber, "uploads/" + tempFileName);
            bladeRedis.expire(serialNumber, 60 * 30);
        }
        return R.data("uploads/" + tempFileName, "uploads/" + tempFileName);
    }

    @PostMapping("/getBySerialNumber")
    @ApiOperation(value = "流水号", notes = "根据流水号获取文件集合 url")
    public R getBySerialNumber(@RequestParam("serialNumber") String serialNumber) {
        if (StringUtils.isBlank(serialNumber)) {
            return R.fail("流水号不能为空");
        }
        SetOperations<String, Object> setOps = bladeRedis.getSetOps();
        Set<Object> members = setOps.members(serialNumber);
        return R.data(members);
    }

    @PostMapping("/delBySerialNumber")
    @ApiOperation(value = "流水号", notes = "根据流水号 删除文件集合 对应文件")
    public R delBySerialNumber(@RequestParam("serialNumber") String serialNumber, @RequestParam("fileUrl") String fileUrl) {
        if (StringUtils.isBlank(serialNumber)) {
            return R.fail("流水号不能为空");
        }
        SetOperations<String, Object> setOps = bladeRedis.getSetOps();
        setOps.remove(serialNumber, fileUrl);
//        Set<Object> members = setOps.members(serialNumber);
//        if(CollectionUtil.isNotEmpty(members)){
//            members.remove(fileUrl);
//        }
        return R.data(setOps.members(serialNumber));
    }

    @PostMapping("/uploadStaticJson")
    @ApiOperation(value = "上传静态json", notes = "上传静态json")
    public R uploadStaticJson(@RequestBody String json) {
        if (StringUtils.isBlank(json)) {
            return R.fail("json不能为空");
        }
        String tempFileName = UUID.randomUUID().toString() + ".json";

        File f = new File(picDir + tempFileName);
        try {
            String decoderStr = new String(Base64.getDecoder().decode(json), "utf-8");
            FileUtil.writeUtf8String(decoderStr, f);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return R.success("uploads/" + tempFileName);
    }


}
